package jp.co.glossom.adfurikun

import android.content.Context
import android.view.View
import android.widget.FrameLayout
import io.flutter.plugin.common.*
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory
import jp.tjkapp.adfurikunsdk.moviereward.*

/**
 * バナー
 */
class AdfurikunBannerFactory(private val messenger: BinaryMessenger, private val channel : MethodChannel) : PlatformViewFactory(StandardMessageCodec.INSTANCE) {

    private val bannerViewMap = hashMapOf<Int, AdfurikunBannerView>()

    override fun create(context: Context?, viewId: Int, args: Any?): PlatformView {
        return AdfurikunBannerView(context, messenger, args, viewId).apply {
            convertViewNumber(args)?.let { number ->
                bannerViewMap[number] = this
            }
        }
    }

    fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        convertViewNumber(call.arguments)?.let { number ->
            bannerViewMap[number]?.onMethodCall(call, result)
        }
    }

    private fun convertViewNumber(args: Any?): Int? {
        val arguments = (args as? HashMap<String, *>)
        return (arguments?.get(Constants.KEY_NUMBER) as? Int)
    }

    inner class AdfurikunBannerView internal constructor(context: Context?, messenger: BinaryMessenger?, args: Any?, private var id: Int)
        : PlatformView, MethodChannel.MethodCallHandler {

        private var banner: AdfurikunBanner? = null
        private var loadListener: AdfurikunBannerLoadListener? = null
        private var playListener: AdfurikunBannerVideoListener? = null

        private var parentView: FrameLayout? = null

        private var number = -1
        private var appId = ""

        init {
            context?.let {
                parentView = FrameLayout(it)
            }
            val arguments = (args as? HashMap<String, *>)
            number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
            appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
            init(args)
        }

        override fun getView(): View? {
            return parentView
        }

        override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
            when (call.method) {
                Constants.INIT_BANNER -> init(call.arguments)
                Constants.LOAD_BANNER -> {
                    if (isTargetAppId(call)) {
                        val timeout = AdfurikunCommons.getLoadWithTimeout(call.arguments as? HashMap<String, *>)
                        if (timeout > 0) {
                            banner?.loadWithTimeout(timeout)
                        } else {
                            banner?.load()
                        }
                    }
                }
                Constants.PLAY_BANNER -> {
                    if (isTargetAppId(call)) {
                        banner?.bannerView?.let {
                            if (it.parent == null) {
                                parentView?.addView(it)
                            }
                        }
                        banner?.play()
                    }
                }
                Constants.DESTROY_BANNER -> {
                    if (isTargetAppId(call)) {
                        dispose()
                    }
                }
                else -> result.notImplemented()
            }
        }

        /**
         * 初期化
         * @param args
         */
        private fun init(args: Any?) {
            val arguments = (args as? HashMap<String, *>)
            val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
            if (appId.isNotBlank() && this.appId == appId) {
                AdfurikunCommons.setPluginVersion(arguments)
                if (banner == null) {
                    var width = 320
                    var height = 50
                    AdfurikunPlugin.currentActivity?.let { activity ->
                        width = AdfurikunCommons.convertDpToPixel(activity, 320).toInt()
                        height = AdfurikunCommons.convertDpToPixel(activity, 50).toInt()
                    }
                    banner = AdfurikunBanner(AdfurikunPlugin.currentActivity, appId, width, height).apply {
                        setAdfurikunBannerLoadListener(createLoadListener)
                        setAdfurikunBannerVideoListener(createPlayListener)
                    }
                }
            }
        }

        /**
         * 対象AppId判定
         * @param call
         */
        private fun isTargetAppId(call: MethodCall): Boolean {
            val arguments = (call.arguments as? HashMap<String, *>)
            val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
            return (appId.isNotBlank() && appId == this.appId)
        }

        override fun dispose() {
            banner?.onDestroy()
            banner = null
            loadListener = null
            playListener = null
            bannerViewMap.remove(number)
        }

        /**
         * 読み込みリスナーを作成
         */
        private val createLoadListener: AdfurikunBannerLoadListener?
            get() {
                loadListener ?: run {
                    loadListener = object : AdfurikunBannerLoadListener {
                        override fun onBannerLoadFinish(adInfo: AdfurikunBannerAdInfo?, appId: String?) {
                            adInfo?.let {
                                channel.invokeMethod(Constants.ON_BANNER_LOAD_FINISH, 
                                        AdfurikunCommons.getArgs(number = number, appId = appId ?: "", adNetworkKey = it.adNetworkKey, title = it.title, description = it.description))
                            }
                        }

                        override fun onBannerLoadError(error: AdfurikunMovieError?, appId: String?) {
                            channel.invokeMethod(Constants.ON_BANNER_LOAD_ERROR, AdfurikunCommons.getArgs(number = number, appId = appId ?: "", errorCode = error?.errorCode ?: 0, adNetworkErrorList = error?.adNetworkErrorList))
                        }
                    }
                }
                return loadListener
            }

        /**
         * 再生リスナーを作成
         */
        private val createPlayListener: AdfurikunBannerVideoListener?
            get() {
                playListener ?: run {
                    playListener = object : AdfurikunBannerVideoListener {
                        override fun onBannerViewPlayStart(appId: String?) {
                            channel.invokeMethod(Constants.ON_BANNER_VIEW_PLAY_START, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                        }

                        override fun onBannerViewPlayFinish(appId: String?, isVideoAd: Boolean) {
                            channel.invokeMethod(Constants.ON_BANNER_VIEW_PLAY_FINISH, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                        }

                        override fun onBannerViewPlayFail(appId: String?, error: AdfurikunMovieError?) {
                            channel.invokeMethod(Constants.ON_BANNER_VIEW_PLAY_FAIL, AdfurikunCommons.getArgs(number = number, appId = appId ?: "", errorCode = error?.errorCode ?: 0, adNetworkErrorList = error?.adNetworkErrorList))
                        }

                        override fun onBannerViewClicked(appId: String?) {
                            channel.invokeMethod(Constants.ON_BANNER_VIEW_CLICKED, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                        }
                    }
                }
                return playListener
            }
    }
}