package jp.co.glossom.adfurikun

import android.content.Context
import android.view.View
import android.widget.FrameLayout
import io.flutter.plugin.common.*
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory
import jp.tjkapp.adfurikunsdk.moviereward.*

/**
 * レクタングル
 */
class AdfurikunRectangleFactory(private val messenger: BinaryMessenger, private val channel : MethodChannel) : PlatformViewFactory(StandardMessageCodec.INSTANCE) {

    private var rectangleViewMap = hashMapOf<Int, AdfurikunRectangleView>()
    
    override fun create(context: Context?, viewId: Int, args: Any?): PlatformView {
        return AdfurikunRectangleView(context, messenger, args, viewId).apply {
            convertViewNumber(args)?.let { number ->
                rectangleViewMap[number] = this
            }
        }
    }

    fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        convertViewNumber(call.arguments)?.let { number ->
            rectangleViewMap[number]?.onMethodCall(call, result)
        }
    }

    private fun convertViewNumber(args: Any?): Int? {
        val arguments = (args as? HashMap<String, *>)
        return (arguments?.get(Constants.KEY_NUMBER) as? Int)
    }

    inner class AdfurikunRectangleView internal constructor(context: Context?, messenger: BinaryMessenger?, args: Any?, private var id: Int)
        : PlatformView, MethodChannel.MethodCallHandler {

        private var rectangle: AdfurikunRectangle? = null
        private var loadListener: AdfurikunRectangleLoadListener? = null
        private var playListener: AdfurikunRectangleVideoListener? = null

        private var parentView: FrameLayout? = null

        private var number = -1
        private var appId = ""

        init {
            context?.let {
                parentView = FrameLayout(it)
            }
            val arguments = (args as? HashMap<String, *>)
            number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
            appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
            init(args)
        }

        override fun getView(): View? {
            return parentView
        }

        override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
            when (call.method) {
                Constants.INIT_RECTANGLE -> init(call.arguments)
                Constants.LOAD_RECTANGLE -> {
                    if (isTargetAppId(call)) {
                        val timeout = AdfurikunCommons.getLoadWithTimeout(call.arguments as? HashMap<String, *>)
                        if (timeout > 0) {
                            rectangle?.loadWithTimeout(timeout)
                        } else {
                            rectangle?.load()
                        }
                    }
                }
                Constants.PLAY_RECTANGLE -> {
                    if (isTargetAppId(call)) {
                        rectangle?.rectangleView?.let {
                            if (it.parent == null) {
                                parentView?.addView(it)
                            }
                        }
                        rectangle?.play()
                    }
                }
                Constants.DESTROY_RECTANGLE -> {
                    if (isTargetAppId(call)) {
                        dispose()
                    }
                }
                else -> result.notImplemented()
            }
        }

        /**
         * 初期化
         * @param args
         */
        private fun init(args: Any?) {
            val arguments = (args as? HashMap<String, *>)
            val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
            AdfurikunCommons.setPluginVersion(arguments)
            if (appId.isNotBlank() && this.appId == appId) {
                if (rectangle == null) {
                    var width = 300
                    var height = 250
                    AdfurikunPlugin.currentActivity?.let { activity ->
                        width = AdfurikunCommons.convertDpToPixel(activity, 300).toInt()
                        height = AdfurikunCommons.convertDpToPixel(activity, 250).toInt()
                    }
                    rectangle = AdfurikunRectangle(AdfurikunPlugin.currentActivity, appId, width, height).apply {
                        setAdfurikunRectangleLoadListener(createLoadListener)
                        setAdfurikunRectangleVideoListener(createPlayListener)
                    }
                }
            }
        }

        /**
         * 対象AppId判定
         * @param call
         */
        private fun isTargetAppId(call: MethodCall): Boolean {
            val arguments = (call.arguments as? HashMap<String, *>)
            val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
            return (appId.isNotBlank() && appId == this.appId)
        }

        override fun dispose() {
            rectangle?.onDestroy()
            rectangle = null
            loadListener = null
            playListener = null
            rectangleViewMap.remove(number)
        }

        /**
         * 読み込みリスナーを作成
         */
        private val createLoadListener: AdfurikunRectangleLoadListener?
            get() {
                loadListener ?: run {
                    loadListener = object : AdfurikunRectangleLoadListener {
                        override fun onRectangleLoadFinish(adInfo: AdfurikunRectangleAdInfo?, appId: String?) {
                            adInfo?.let {
                                channel.invokeMethod(Constants.ON_RECTANGLE_LOAD_FINISH,
                                        AdfurikunCommons.getArgs(number = number, appId = appId ?: "", adNetworkKey = it.adNetworkKey, title = it.title, description = it.description))
                            }
                        }

                        override fun onRectangleLoadError(error: AdfurikunMovieError?, appId: String?) {
                            channel.invokeMethod(Constants.ON_RECTANGLE_LOAD_ERROR, AdfurikunCommons.getArgs(number = number, appId = appId ?: "", errorCode = error?.errorCode ?: 0, adNetworkErrorList = error?.adNetworkErrorList))
                        }
                    }
                }
                return loadListener
            }

        /**
         * 再生リスナーを作成
         */
        private val createPlayListener: AdfurikunRectangleVideoListener?
            get() {
                playListener ?: run {
                    playListener = object : AdfurikunRectangleVideoListener {
                        override fun onRectangleViewPlayStart(appId: String?) {
                            channel.invokeMethod(Constants.ON_RECTANGLE_VIEW_PLAY_START, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                        }

                        override fun onRectangleViewPlayFinish(appId: String?, isVideoAd: Boolean) {
                            channel.invokeMethod(Constants.ON_RECTANGLE_VIEW_PLAY_FINISH, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                        }

                        override fun onRectangleViewPlayFail(appId: String?, error: AdfurikunMovieError?) {
                            channel.invokeMethod(Constants.ON_RECTANGLE_VIEW_PLAY_FAIL, AdfurikunCommons.getArgs(number = number, appId = appId ?: "", errorCode = error?.errorCode ?: 0, adNetworkErrorList = error?.adNetworkErrorList))
                        }

                        override fun onRectangleViewClicked(appId: String?) {
                            channel.invokeMethod(Constants.ON_RECTANGLE_VIEW_CLICKED, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                        }
                    }
                }
                return playListener
            }
    }
}