import Foundation
import Flutter
import ADFMovieReward

// MARK: InfeedAdView Class

class InfeedAdView: NSObject, FlutterPlatformView, ADFmyNativeAdDelegate, ADFMediaViewDelegate {
    private let number: Int
    private let channel: FlutterMethodChannel
    private let containerView: UIView
    private var infeedAd: ADFmyNativeAd?
    private var infeedAdInfo: ADFNativeAdInfo?
    private var appId: String = ""
    private var adType: String = ""
    
    init(_ number: Int, _ channel: FlutterMethodChannel) {
        self.number = number
        self.channel = channel
        containerView = UIView()
        containerView.backgroundColor = .clear
    }
    
    func view() -> UIView {
        return containerView
    }
    
    func adInit(_ appId: String, _ adType: String, _ width: Double, _ height: Double, _ pluginVersion: String?) {
        self.appId = appId
        self.adType = adType
        containerView.frame = CGRect(x: 0, y: 0, width: width, height: height)
        
        let option: [AnyHashable: Any] = [ADFConst.KEY_PLUGIN:
                                            [ADFConst.KEY_TYPE: ADFConst.VALUE_FLUTTER,
                                             ADFConst.KEY_VERSION: pluginVersion ?? ""]]
        if ADFConst.AD_TYPE_NATIVEAD == adType {
            infeedAd = ADFmyNativeAd.createInstance(appId, option: option)
        } else if ADFConst.AD_TYPE_RECTANGLE == adType {
            infeedAd = ADFmyRectangle.createInstance(appId, option: option)
        } else if ADFConst.AD_TYPE_BANNER == adType {
            infeedAd = ADFmyBanner.createInstance(appId, option: option)
        }
    }
    
    func setAdLoadingTimeout(_ timeout: Int32) {
        infeedAd?.setLoadingTimeout(timeout)
    }
    
    func adLoad() {
        infeedAd?.loadAndNotify(to: self)
    }
    
    func adPlay() {
        if let mediaView = infeedAdInfo?.mediaView {
            mediaView.frame = containerView.frame
            containerView.addSubview(mediaView)
            mediaView.mediaViewDelegate = self
            infeedAdInfo?.setCustomMediaView(containerView)
            infeedAdInfo?.playMediaView()
        }
    }
    
    func adDestroy() {
        infeedAdInfo?.mediaView?.removeFromSuperview()
        infeedAd?.dispose()
        InfeedAdViewHolder.remove(number)
    }
    
    private func args(number: Int,
                      appId: String,
                      adNetworkKey: String? = nil,
                      title: String? = nil,
                      description: String? = nil,
                      errorCode: Int? = nil,
                      adNetworkErrorList: [[String: Any]]? = nil) -> Dictionary<String, Any?> {
        return [
            ADFConst.KEY_NUMBER: number,
            ADFConst.KEY_APP_ID: appId,
            ADFConst.KEY_AD_NETWORK_KEY: adNetworkKey,
            ADFConst.KEY_TITLE: title,
            ADFConst.KEY_DESCRIPTION: description,
            ADFConst.KEY_ERROR_CODE: errorCode,
            ADFConst.KEY_AD_NETWORK_ERROR: adNetworkErrorList
        ]
    }
    
    private func invokeMethod(_ method: String, _ args: Dictionary<String, Any?>) {
        channel.invokeMethod(method, arguments: args)
    }
    
    // MARK: ADF Delegate Method
    
    func onNativeAdLoadFinish(_ info: ADFNativeAdInfo, appID: String) {
        print("[ADF DEBUG] call onNativeAdLoadFinish, appId:\(appId), number:\(number)")
        infeedAdInfo = info
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appId,
                                                  adNetworkKey: info.adnetworkKey,
                                                  title: info.title,
                                                  description: info.description)
        if adType == ADFConst.AD_TYPE_NATIVEAD {
            invokeMethod(ADFConst.CALLBACK_ON_NATIVEAD_LOAD_FINISH, args)
        } else if adType ==  ADFConst.AD_TYPE_RECTANGLE {
            invokeMethod(ADFConst.CALLBACK_ON_RECTANGLE_LOAD_FINISH, args)
        } else if adType == ADFConst.AD_TYPE_BANNER {
            invokeMethod(ADFConst.CALLBACK_ON_BANNER_LOAD_FINISH, args)
        }
    }
    
    func onNativeAdLoadError(_ error: ADFMovieError, appID: String, adnetworkError: [AdnetworkError]) {
        print("[ADF DEBUG] call onNativeAdLoadError, appId:\(appId), number:\(number), error:\(error.errorMessage ?? "nil")")
        var errorCode: Int? = nil
        switch error.errorCode {
        case .noStock:
            errorCode = 0
            break
        case .networkDisconnect:
            errorCode = 1
            break
        //.loading = 2
        case .invalidAppId:
            errorCode = 3
            break
        case .apiRequestFailure:
            errorCode = 4
            break
        case .unsupportedOSVersion:
            errorCode = 5
            break
        case .playerItemLoadFailure:
            errorCode = 6
            break
        case .playerItemPlayFailure:
            errorCode = 7
            break
        case .other:
            errorCode = 8
            break
        default:
            break
        }
        var adnetworkErrorList: [[String: Any]] = []
        for e in adnetworkError {
            var error: [String: Any] = [:]
            error[ADFConst.KEY_AD_NETWORK_KEY] = e.adnetworkKey
            error[ADFConst.KEY_ERROR_CODE] = e.errorCode
            error[ADFConst.KEY_ERROR_MESSAGE] = e.errorMessage
            adnetworkErrorList.append(error)
        }
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appId,
                                                  errorCode: errorCode,
                                                  adNetworkErrorList: adnetworkErrorList)
        if adType == ADFConst.AD_TYPE_NATIVEAD {
            invokeMethod(ADFConst.CALLBACK_ON_NATIVEAD_LOAD_ERROR, args)
        } else if adType ==  ADFConst.AD_TYPE_RECTANGLE {
            invokeMethod(ADFConst.CALLBACK_ON_RECTANGLE_LOAD_ERROR, args)
        } else if adType == ADFConst.AD_TYPE_BANNER {
            invokeMethod(ADFConst.CALLBACK_ON_BANNER_LOAD_ERROR, args)
        }
    }
    
    func onADFMediaViewPlayStart() {
        print("[ADF DEBUG] call onADFMediaViewPlayStart, appId:\(appId), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appId)
        if adType == ADFConst.AD_TYPE_NATIVEAD {
            invokeMethod(ADFConst.CALLBACK_ON_NATIVEAD_VIEW_PLAY_START, args)
        } else if adType ==  ADFConst.AD_TYPE_RECTANGLE {
            invokeMethod(ADFConst.CALLBACK_ON_RECTANGLE_VIEW_PLAY_START, args)
        } else if adType == ADFConst.AD_TYPE_BANNER {
            invokeMethod(ADFConst.CALLBACK_ON_BANNER_VIEW_PLAY_START, args)
        }
    }
    
    func onADFMediaViewPlayFinish() {
        print("[ADF DEBUG] call onADFMediaViewPlayFinish, appId:\(appId), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appId)
        if adType == ADFConst.AD_TYPE_NATIVEAD {
            invokeMethod(ADFConst.CALLBACK_ON_NATIVEAD_VIEW_PLAY_FINISH, args)
        } else if adType ==  ADFConst.AD_TYPE_RECTANGLE {
            invokeMethod(ADFConst.CALLBACK_ON_RECTANGLE_VIEW_PLAY_FINISH, args)
        } else if adType == ADFConst.AD_TYPE_BANNER {
            invokeMethod(ADFConst.CALLBACK_ON_BANNER_VIEW_PLAY_FINISH, args)
        }
    }
    
    func onADFMediaViewClick() {
        print("[ADF DEBUG] call onADFMediaViewClick, appId:\(appId), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appId)
        if adType == ADFConst.AD_TYPE_NATIVEAD {
            invokeMethod(ADFConst.CALLBACK_ON_NATIVEAD_VIEW_CLICKED, args)
        } else if adType ==  ADFConst.AD_TYPE_RECTANGLE {
            invokeMethod(ADFConst.CALLBACK_ON_RECTANGLE_VIEW_CLICKED, args)
        } else if adType == ADFConst.AD_TYPE_BANNER {
            invokeMethod(ADFConst.CALLBACK_ON_BANNER_VIEW_CLICKED, args)
        }
    }
    
    func onADFMediaViewPlayFail() {
        print("[ADF DEBUG] call onADFMediaViewPlayFail, appId:\(appId), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appId)
        if adType == ADFConst.AD_TYPE_NATIVEAD {
            invokeMethod(ADFConst.CALLBACK_ON_NATIVEAD_VIEW_PLAY_FAIL, args)
        } else if adType ==  ADFConst.AD_TYPE_RECTANGLE {
            invokeMethod(ADFConst.CALLBACK_ON_RECTANGLE_VIEW_PLAY_FAIL, args)
        } else if adType == ADFConst.AD_TYPE_BANNER {
            invokeMethod(ADFConst.CALLBACK_ON_BANNER_VIEW_PLAY_FAIL, args)
        }
    }
}

// MARK: InfeedAdViewHolder Class

class InfeedAdViewHolder {
    static private var viewMap: Dictionary<Int, InfeedAdView> = [:]
    
    static func createIfNeeded(_ channel: FlutterMethodChannel, _ number: Int) -> InfeedAdView {
        if let infeedAdView: InfeedAdView = InfeedAdViewHolder.viewMap[number] {
            return infeedAdView
        } else {
            let infeedAdView: InfeedAdView = InfeedAdView(number, channel)
            InfeedAdViewHolder.viewMap.updateValue(infeedAdView, forKey: number)
            return infeedAdView
        }
    }
    
    static func get(_ number: Int) -> InfeedAdView? {
        return viewMap[number]
    }
    
    static func remove(_ number: Int) {
        viewMap.removeValue(forKey: number)
    }
}

// MARK: InfeedAdViewFactory Class

class InfeedAdViewFactory: NSObject, FlutterPlatformViewFactory {
    private let channel: FlutterMethodChannel
    
    init(_ channel: FlutterMethodChannel) {
        self.channel = channel
    }
    
    func create(withFrame frame: CGRect, viewIdentifier viewId: Int64, arguments args: Any?) -> FlutterPlatformView {
        guard let args = args as? [String: Any?],
              let number = args[ADFConst.KEY_NUMBER] as? Int else {
            print("[ADF WARNING] InfeedAdViewFactory:create args is abnormal")
            return InfeedAdView(-1, channel)
        }
        let infeedAdView = InfeedAdViewHolder.createIfNeeded(channel, number)
        return infeedAdView
    }
    
    func createArgsCodec() -> FlutterMessageCodec & NSObjectProtocol {
        return FlutterStandardMessageCodec.sharedInstance()
    }
}

// MARK: InfeedAdManager Class

class InfeedAdManager {
    private let channel: FlutterMethodChannel
    
    init(_ channel: FlutterMethodChannel, _ register: FlutterPluginRegistrar) {
        self.channel = channel
        let viewFactory = InfeedAdViewFactory(channel)
        register.register(viewFactory, withId: ADFConst.ADFURIKUN_VIEW_TYPE_BASE + ADFConst.AD_TYPE_NATIVEAD)
        register.register(viewFactory, withId: ADFConst.ADFURIKUN_VIEW_TYPE_BASE + ADFConst.AD_TYPE_RECTANGLE)
        register.register(viewFactory, withId: ADFConst.ADFURIKUN_VIEW_TYPE_BASE + ADFConst.AD_TYPE_BANNER)
    }
    
    func checkMethod(_ call: FlutterMethodCall, _ checkString: String) -> Bool {
        return call.method.hasPrefix(checkString + ADFConst.SEPARATOR_METHOD)
    }
    
    func handle(_ call: FlutterMethodCall, _ result: @escaping FlutterResult, _ adType: String) {
        guard let args = call.arguments as? [String: Any],
              let number = args[ADFConst.KEY_NUMBER] as? Int,
              let appId = args[ADFConst.KEY_APP_ID] as? String else {
            print("[ADF WARNING] InfeedAdManager:handle args is abnormal")
            return
        }
        if checkMethod(call, ADFConst.METHOD_INIT) {
            let pluginVersion: String? = args[ADFConst.KEY_PLUGIN_VERSION] as? String
            var width: Double? = args[ADFConst.KEY_WIDTH] as? Double
            var height: Double? = args[ADFConst.KEY_HEIGHT] as? Double
            if width == nil || height == nil {
                print("[ADF WARNING] InfeedAdManager:handle width,height is abnormal")
                if ADFConst.AD_TYPE_NATIVEAD == adType {
                    width = 320
                    height = 180
                } else if ADFConst.AD_TYPE_BANNER == adType {
                    width = 320
                    height = 50
                } else {
                    width = 300
                    height = 250
                }
            }
            if let adWidth = width, let adHeight = height {
                let infeedAdView = InfeedAdViewHolder.createIfNeeded(channel, number)
                infeedAdView.adInit(appId, adType, adWidth, adHeight, pluginVersion)
            }
        } else if checkMethod(call, ADFConst.METHOD_LOAD) {
            if let infeedAdView = InfeedAdViewHolder.get(number) {
                if let timeout = args[ADFConst.KEY_TIMEOUT] as? Int32, timeout > 0 {
                    infeedAdView.setAdLoadingTimeout(timeout)
                    infeedAdView.adLoad()
                } else {
                    infeedAdView.adLoad()
                }
            }
        } else if checkMethod(call, ADFConst.METHOD_PLAY) {
            if let infeedAdView = InfeedAdViewHolder.get(number) {
                infeedAdView.adPlay()
            }
        } else if checkMethod(call, ADFConst.METHOD_DESTROY) {
            if let infeedAdView = InfeedAdViewHolder.get(number) {
                infeedAdView.adDestroy()
            }
        }
    }
}
